import logging
import time
import json
import socket
from datetime import datetime, timedelta
from pymongo.cursor import Cursor
from bson.objectid import ObjectId
from flask import abort, Response


logging.basicConfig(
    level=logging.WARNING,
    format='[%(asctime)s] [%(levelname)s] %(message)s'
)


def handle_date(date: str, date_type='start') -> datetime:
    try:
        if not date:
            date_data = None
        elif date.isdigit():
            date_time = date[:10]
            date_data = datetime.fromtimestamp(int(date_time))
        else:
            date_data = datetime.strptime(date, "%Y-%m-%d")
            if date_type == 'end' and date_data:
                date_data += timedelta(hours=23, minutes=59, seconds=59)
        return date_data
    except Exception as err:
        handle_abnormal(
            message='日期字符串解析异常',
            status=400,
            other={'prompt': str(err)}
        )


def handle_db_to_list(db_cursor: Cursor) -> list:
    """
    将db列表数据，转换为list（原db的id是ObjectId类型，转为json会报错）
    :param db_cursor:db列表数据（find()获取）
    :return:转换后的列表
    """
    if not isinstance(db_cursor, Cursor):
        handle_abnormal(
            message='类型必须是 %s (获取类型为 %s)' % (Cursor, type(db_cursor)),
            status=500,
        )
    result = []
    for db in db_cursor:
        result.append(handle_db_dict(db_dict=db))
    return result


def handle_db_dict(db_dict: dict) -> dict:
    try:
        db_dict['id'] = str(db_dict['_id'])
        del db_dict['_id']
        # 创建时间和更新时间如果不存在，则设置为当前时间
        db_dict['creation_time'] = db_dict.get(
            'creation_time', datetime.today())
        db_dict['update_time'] = db_dict.get('update_time', datetime.today())
        # 将日期格式的时间，转化为13位时间戳
        db_dict['creation_time'] = int(time.mktime(
            db_dict['creation_time'].timetuple())) * 1000
        db_dict['update_time'] = int(time.mktime(
            db_dict['update_time'].timetuple())) * 1000
        return db_dict
    except Exception as err:
        handle_abnormal(
            message='MongoDB 数据字典解析异常',
            status=500,
            other={'prompt': str(err)}
        )


def handle_db_id(data_dict: dict) -> dict:
    if not isinstance(data_dict, dict):
        handle_abnormal(
            message='类型错误，预期 %s ,却得到 %s' % (dict, type(id)),
            status=400,
        )
    try:
        if data_dict.__contains__('_id') and isinstance(data_dict['_id'], str):
            data_dict['_id'] = ObjectId(str(data_dict['_id']))
        elif data_dict.__contains__('id'):
            if isinstance(data_dict['id'], str):
                data_dict['_id'] = ObjectId(str(data_dict['id']))
            else:
                data_dict['_id'] = data_dict['id']
            del data_dict['id']
        # 处理批量操作时，字符串类型ID问题
        # 判断存在_id，并且值为字典则做特殊处理
        if data_dict.__contains__('_id') and isinstance(data_dict['_id'], dict):
            for key, value in data_dict['_id'].items():
                if isinstance(value, list):  # 查找ID是列表
                    for i in range(len(value)):
                        if isinstance(value[i], str):  # 是字符串
                            data_dict['_id'][key][i] = ObjectId(str(value[i]))
                elif isinstance(value, str):       # 支持mongo的高级查询 {'_id': {'$ne': ObjectId('60867c4b78254307721d4617')}}
                    data_dict['_id'][key] = ObjectId(str(value))
        return data_dict
    except Exception as err:
        handle_abnormal(
            message='MongoDB _id 格式解析异常',
            status=400,
            other={'prompt': str(err)}
        )


def handle_db_remove(find_dict: dict) -> dict:
    try:
        if not find_dict.__contains__('remove_time'):
            # 设置搜索条件为不存在remove_time字段的文件
            find_dict['remove_time'] = {'$exists': False}
        # 返回设置后的搜索条件
        return find_dict
    except Exception as err:
        handle_abnormal(
            message='MongoDB 移除数据处理异常',
            status=500,
            other={'prompt': str(err)}
        )


def handle_abnormal(message: str, status: int, header: dict = {'Content-Type': 'application/json'}, other: dict = {}, is_raw: bool = False):
    logging.warning(message)
    host_name = socket.gethostname()
    server_time = datetime.today()
    return_body = {
        'code': status,
        'message': message,
        'server_time': int(server_time.strftime('%Y%m%d%H%M%S')) * 1000,
        'host_name': host_name,
        'host_ip': socket.gethostbyname(host_name),
    }
    return_body.update(other)
    if is_raw:
        return return_body, status, header
    else:
        abort(Response(json.dumps(return_body), status, header))


def handle_time_difference(start_timestamp: int, end_timestamp: int) -> float:
    if len(str(start_timestamp)) == 13:
        start_date = datetime.fromtimestamp(start_timestamp/1000)
        end_date = datetime.fromtimestamp(end_timestamp/1000)
    elif len(str(start_timestamp)) == 10:
        start_date = datetime.fromtimestamp(start_timestamp)
        end_date = datetime.fromtimestamp(end_timestamp)
    else:
        handle_abnormal(message='时间戳格式不规范', status=400,)

    difference = (end_date - start_date).total_seconds()
    return difference



import os
import zipfile
from io import BytesIO
from .tools import handle_abnormal
from flask import Response, send_from_directory, make_response
from urllib.parse import quote


def download_file(file_path: str, file_name: str):
    if not os.path.exists(os.path.join(file_path, file_name)):
        handle_abnormal(
            message='文件(普通下载)路径不存在({0})'.format(
                os.path.join(file_path, file_name)),
            status=400,
            other={'file_path': file_path, 'file_name': file_name}
        )
    response = make_response(send_from_directory(
        file_path,  # 本地目录的path
        file_name,  # 文件名(带扩展名)
        as_attachment=True
    ))
    # 确保文件名称包含中文，逗号等各种符号时均能正常响应
    response.headers['Content-Disposition'] = "attachment; filename={0}; filename*=utf-8''{0}".format(
        quote(file_name))

    return response


def download_flow(file_path: str, file_name: str):
    complete_path = os.path.join(file_path, file_name)
    if not os.path.exists(complete_path):
        handle_abnormal(
            message='文件(流传输)路径不存在',
            status=400,
            other={'complete_path': complete_path}
        )

    def send_file():
        # 流式读取文件
        with open(complete_path, 'rb') as target_file:
            while 1:
                data = target_file.read(20 * 1024 * 1024)  # 每次读取20M
                if not data:
                    break
                yield data

    # 浏览器不识别的也会自动下载
    response = Response(send_file(), content_type='application/octet-stream')
    # 确保文件名称包含中文，逗号等各种符号时均能正常响应
    response.headers['Content-Disposition'] = "attachment; filename={0}; filename*=utf-8''{0}".format(
        quote(file_name))
    return response


def download_directory(dir_path: str, zip_name: str, save_zip=False):
    """ 将目录打包zip 返回数据"""
    if not os.path.exists(dir_path):
        handle_abnormal(
            message='目录路径不存在',
            status=400,
            other={'dir_path': dir_path}
        )
    # 创建一个内存对象，用于保存二进制文件
    memory_zip = BytesIO()
    # 在内存中打开一个压缩文件
    with zipfile.ZipFile(memory_zip, 'w', zipfile.ZIP_DEFLATED) as zf:
        # 将数据存储到内存的zip文件中
        for r, m, f in os.walk(dir_path):
            fpath = r.replace(dir_path, '')
            for _f in f:
                zf.write(os.path.join(r, _f), os.path.join(fpath, _f))
        for zfile in zf.filelist:
            zfile.create_system = 0
    # 读文件前，要将位置置位初始，从头开始读取文件
    memory_zip.seek(0)
    # 将内存中的数据读取出来
    data = memory_zip.read()
    # 关闭内存
    memory_zip.close()
    # 需要本地保存zip文件
    if save_zip:
        parent_path = os.path.dirname(dir_path)
        save_path = os.path.join(parent_path, zip_name)
        with open(save_path, 'wb') as z:
            z.write(data)
    # 浏览器不识别的也会自动下载
    response = Response(data, content_type='application/zip')
    # 确保文件名称包含中文，逗号等各种符号时均能正常响应
    response.headers['Content-Disposition'] = "attachment; filename={0}; filename*=utf-8''{0}".format(
        quote(zip_name))
    return response



import os
import json
import pymongo
import logging
import math
import copy
from datetime import datetime
from .parameter import Parameter
from .tools import handle_date, handle_db_to_list, handle_db_dict, handle_db_id, handle_db_remove, handle_abnormal
from flask import abort, Response
from bson import ObjectId

logging.basicConfig(
    level=logging.WARNING,
    format='[%(asctime)s] [%(levelname)s] %(message)s'
)


class DBConnect:
    def __init__(self, db: str, collection: str, home_key='MONGODB_HOST', port_key='MONGODB_PORT'):
        self.mongo_home = os.environ.get(home_key, None)
        self.mongo_port = os.environ.get(port_key, 27017)

        if not self.mongo_home or not self.mongo_port:
            logging.error('MongoDB(组件)的组件连接信息是不完整的')
        else:
            logging.warning('已连接地址为 {0}:{1} 的 MongoDB(组件)'.format(
                self.mongo_home, self.mongo_port))

        self.mongo_client = pymongo.MongoClient(
            host=self.mongo_home,
            port=int(self.mongo_port)
        )
        self.mongo_db = self.mongo_client[db]
        self.mongo_collection = self.mongo_db[collection]

    def write_one_docu(self, docu: dict) -> str:
        try:
            deep_docu = copy.deepcopy(docu)  # 深拷贝
            # 兼容直接指定ID的创建方式
            deep_docu = handle_db_id(deep_docu)
            # 添加创建时间和更新时间，日期格式
            if not deep_docu.__contains__('creation_time'):
                deep_docu['creation_time'] = datetime.today()
            if not deep_docu.__contains__('update_time'):
                deep_docu['update_time'] = datetime.today()
            new_data = self.mongo_collection.insert_one(deep_docu)
            return str(new_data.inserted_id)
        except Exception as err:
            handle_abnormal(
                message='MongoDB(组件)出现写入错误: {0}'.format(json.dumps(docu)),
                status=500,
                other={'prompt': str(err)}
            )

    def write_many_docu(self, docu_list: list) -> list:
        try:
            new_docu_list = []
            for docu in docu_list:
                if isinstance(docu, dict):
                    # 添加创建时间和更新时间，日期格式
                    if not docu.__contains__('creation_time'):
                        docu['creation_time'] = datetime.today()
                    if not docu.__contains__('update_time'):
                        docu['update_time'] = datetime.today()
                    # 深渊巨坑，字段append()到列表中要用深拷贝
                    deep_docu = copy.deepcopy(docu)
                    new_docu_list.append(deep_docu)
                else:
                    handle_abnormal(
                        message='参数docu_list格式不正确，要求为：[{},{},{}...]',
                        status=500,
                        other={'prompt': {'docu_list': docu_list}}
                    )
            new_data_list = self.mongo_collection.insert_many(new_docu_list)
            new_id_list = [str(_id) for _id in new_data_list.inserted_ids]
            return new_id_list
        except Exception as err:
            handle_abnormal(
                message='MongoDB(组件)出现写入错误',
                status=500,
                other={'prompt': str(err)}
            )

    def does_it_exist(self, docu: dict) -> bool:
        deep_docu = copy.deepcopy(docu)  # 深拷贝
        deep_docu = handle_db_id(deep_docu)
        count = self.mongo_collection.count_documents(deep_docu)
        if count != 0:
            return True
        else:
            return False

    def update_docu(self, find_docu: dict, modify_docu: dict, many=False) -> dict:
        try:
            # 将参数id处理成符合mongo格式的_id对象
            find_docu = handle_db_id(find_docu)
            # 排除假删除数据
            find_docu = handle_db_remove(find_docu)
            # 重置更新时间
            if not modify_docu.__contains__('update_time'):
                modify_docu['update_time'] = datetime.today()
            # 将更新内容转化为mongo能识别的更新格式
            modify_docu = {'$set': modify_docu}
            if many:
                student = self.mongo_collection.update_many(
                    find_docu, modify_docu)
            else:
                student = self.mongo_collection.update_one(
                    find_docu, modify_docu)
            # 返回匹配条数、受影响条数
            return {'matched_count': student.matched_count, 'modified_count': student.modified_count}
        except Exception as err:
            handle_abnormal(
                message='MongoDB(组件)出现更新错误',
                status=500,
                other={'prompt': str(err)}
            )

    def delete_docu(self, find_docu: dict, many: bool = False, false_delete: bool = False) -> dict:
        try:
            if false_delete:
                # 假删除流程
                modify_dict = {'remove_time': datetime.today()}
                result = self.update_docu(
                    find_docu=find_docu, modify_docu=modify_dict, many=many)
                if not result['modified_count']:
                    logging.warning('MongoDB(组件)出现删除异常: 没有任何文档被假删除')
                return {'deleted_count': result['modified_count'], 'false_delete': false_delete}
            else:
                find_docu = handle_db_id(find_docu)
                find_docu = handle_db_remove(find_docu)
                # 真删除流程
                if many:
                    # 文档批量删除方式
                    result = self.mongo_collection.delete_many(find_docu)
                else:
                    # 文档精确删除方式
                    result = self.mongo_collection.delete_one(find_docu)
                if not result.deleted_count:
                    logging.error('MongoDB(组件)出现删除异常: 没有任何文档被真删除')
                # 返回删除成功条数
                return {'deleted_count': result.deleted_count, 'false_delete': false_delete}
        except Exception as err:
            handle_abnormal(
                message='MongoDB(组件)出现删除错误',
                status=500,
                other={'prompt': str(err)}
            )

    def find_docu(self, find_dict: dict, many: bool = True) -> list:
        try:
            deep_find_dict = copy.deepcopy(find_dict)  # 深拷贝
            deep_find_dict = handle_db_id(deep_find_dict)
            deep_find_dict = handle_db_remove(deep_find_dict)
            if many:
                # 多文档查询方式
                query_cursor = self.mongo_collection.find(deep_find_dict)
                return handle_db_to_list(query_cursor)
            else:
                # 单文档查询方式
                query_dict = self.mongo_collection.find_one(deep_find_dict)
                if not query_dict:
                    return []
                return [handle_db_dict(query_dict)]
        except Exception as err:
            handle_abnormal(
                message='MongoDB(组件)出现查询错误',
                status=500,
                other={'prompt': str(err)}
            )

    def find_docu_by_id(self, id: str, raise_err=True) -> dict:
        """
        根据id查找记录
        :param id:记录id
        :param raise_err:是否抛出异常（使用abort抛出），否则返回None
        :return: 将结果转换为字典
        """
        if not isinstance(id, str):
            handle_abnormal(
                message='类型错误, 预期 %s ,却得到 %s' % (str, type(id)),
                status=400,
            )
        entity = self.find_docu({'id': id}, many=False)
        if entity is None or len(entity) == 0:
            if raise_err:
                handle_abnormal(
                    message='找不到 id= %s 的记录' % (id,),
                    status=400,
                )
            else:
                return {}
        return entity[0]

    def find_docu_by_id_list(self, id_list: list) -> list:
        """
        根据id列表查找记录列表
        :param id_list:
        :return:
        """
        if not isinstance(id_list, list):
            handle_abnormal(
                message='类型错误, 预期 %s ,却得到 %s' % (list, type(id_list)),
                status=400,
            )
        id_search = []
        for id in id_list:
            id_search.append({'_id': ObjectId(id)})
        find_dict = {'$or': id_search}
        data = self.find_docu(find_dict)
        return data

    def find_paging(self, parameter: Parameter) -> dict:
        # 兼容GET和POST两个请求方式
        if parameter.method == 'GET':
            checking = parameter.param_url
        else:
            checking = parameter.param_json
        param = parameter.verification(
            checking=checking,
            verify={
                '$limit': int, '$offset': int, '$orderby': str, '$start_date': str, '$date_type': str,
                '$end_date': str,
            },
            optional={'$start_date': '', '$end_date': '', '$date_type': 'update_time',
                      '$offset': 0, '$limit': 10, '$orderby': ''},
            allow_extra=True
        )
        try:
            limit = int(param['$limit'])  # 指示页大小（从1开始）
            offset = int(param['$offset'])  # 指示记录起始位置（从0开始）
            if limit < 1:
                raise Exception('指示页大小必须从 1 开始计算')
            # 排序规则（`key1 desc,key2 asc`），asc=升序，desc=降序
            orderby = param['$orderby']
            sort_list = []  # 数据库排序规则列表
            if orderby:
                # 整理请求参数中的筛排序列表
                orderby = orderby.split(',')
                for order in orderby:
                    order_kv = order.split(' ')
                    if len(order_kv) == 2 and (order_kv[1] == 'asc' or order_kv[1] == 'desc') and order_kv[0].strip() != '':
                        if order_kv[1] == 'asc':
                            sort_list.append((order_kv[0], pymongo.ASCENDING))
                        elif order_kv[1] == 'desc':
                            sort_list.append((order_kv[0], pymongo.DESCENDING))
                    else:
                        raise Exception(
                            '排序参数不符合格式要求: "{0}"'.format(param['$orderby']))
            start_date = handle_date(date=param['$start_date'])  # 可选——开始日期
            # 可选——结束日期，与开始日期组成日期区间
            end_date = handle_date(date=param['$end_date'], date_type='end')
            # 查找假删除数据的数据量
            remove_count = self.mongo_collection.find(
                {'remove_time': {'$exists': True}}).count()
            # 查找字典和排序列表，处理假删除数据
            find_dict = handle_db_remove({})
            redundant_filter = param['redundant_dict']  # 额外的数据库筛选字典
            for filter_key, filter_value in redundant_filter.items():
                # 整理请求参数中的筛选项字典
                if type(filter_value) == int or type(filter_value) == dict:
                    find_dict[filter_key] = filter_value
                elif filter_key == 'id' or filter_key == '_id':
                    find_dict['_id'] = ObjectId(str(filter_value))
                elif type(filter_value) == list:
                    # 值为列表，则使用元素匹配逻辑，将数据库中对应字段包含list元素的数据全部赛选出来
                    find_dict[filter_key] = {"$elemMatch":{"$in": filter_value}}
                else:
                    # 查询值模糊匹配的结果
                    if filter_value.strip() != '':
                        find_dict[filter_key] = {'$regex': filter_value}
            # 仅存在日期查询区间参数时，进行额外的创建时间区间查询
            if start_date and end_date:
                find_dict[param['$date_type']] = {
                    '$gte': start_date, '$lte': end_date}
            # 对排序规则处理
            if sort_list:
                find_data = self.mongo_collection.find(find_dict).sort(sort_list).limit(
                    limit).skip(offset)
            else:
                find_data = self.mongo_collection.find(find_dict).limit(
                    limit).skip(offset)
            # 处理返回数据
            records_filtered = find_data.count()
            # 判断数据是否为空
            if records_filtered:
                # 处理返回数据
                query_result = handle_db_to_list(find_data)
                return {
                    'total': records_filtered,
                    'items': query_result,
                    'dummy_remove': remove_count,
                }
            else:
                return {
                    'total': 0,
                    'items': [],
                    'dummy_remove': remove_count,
                }
        except Exception as err:
            handle_abnormal(
                message='计算组件分页参数异常',
                status=400,
                other={'prompt': str(err)}
            )



import os
import sys
import zipfile
import argparse
from pathlib import Path


def main():
    if len(sys.argv) == 1:
        sys.argv.append('--help')
    parser = argparse.ArgumentParser(description='Rainbond Python CLI')
    parser.add_argument('-c', '--create', help='Create Rainbond Python Component',
                        action='store', dest='component_name')
    args = parser.parse_args()

    if args.component_name:
        component_root = Path(__file__).resolve().parents[0]
        os.mkdir(args.component_name)
        example_dir = component_root / 'demos' / 'rainbond-python-demo.zip'
        extract(str(example_dir), args.component_name)
        print(
            'Successfully created `{0}` component. Run command:\n$ cd {0}\n$ pip3 install -r requirements.txt\n$ python3 app.py'.format(args.component_name))


def extract(z_file, path):
    """解压缩文件到指定目录"""
    f = zipfile.ZipFile(z_file, 'r')
    for file in f.namelist():
        f.extract(file, path)


if __name__ == '__main__':
    main()



import json
from flask import abort, Response
from .tools import handle_abnormal


class Parameter():
    def __init__(self, request):
        self.request = request
        self.method = request.method  # 请求方法
        self.headers = dict(request.headers)  # 请求头
        # GET方法的取参
        self.param_url = request.args.to_dict()
        # 非GET方法取值
        try:
            self.param_json = json.loads(
                request.get_data(as_text=True)
            )
        except json.decoder.JSONDecodeError:
            self.param_json = {}
        # 表单提交取值
        self.param_form = dict(request.form)

    def get_content(self) -> dict:
        return {
            'method': self.method,
            'headers': self.headers,
            'param_url': self.param_url,
            'param_json': self.param_json,
            'param_form': self.param_form
        }

    def verification(self, checking: dict, verify: dict, optional: dict = {}, null_value: bool = False, allow_extra: bool = False) -> dict:
        for opt_k, opt_v in optional.items():
            # 没有选填参数时，填充预设的默认值
            if not checking.__contains__(opt_k):
                checking[opt_k] = opt_v
        if not set(verify.keys()).issubset(set(checking.keys())):
            json_data = {'All Parameters': {}, 'Optional': optional}
            for k, v in verify.items():
                json_data['All Parameters'][k] = str(v)
            handle_abnormal(
                message='请求参数不完整',
                status=400,
                other={'prompt': json_data}
            )
        redundant_dict = {}  # 未预先定义的额外参数字典
        for _k, _v in checking.items():
            try:
                str_data = None
                if type(_v) != verify[_k]:
                    if verify[_k] == float:
                        # 特殊处理字符串形式 float 类型判断
                        try:
                            checking[_k] = float(_v)
                        except ValueError as err:
                            str_data = f'参数 {_k} 应该符合 {verify[_k]} 格式'
                    elif verify[_k] == int:
                        # 特殊处理字符串形式 int 类型判断
                        try:
                            checking[_k] = int(_v)
                        except ValueError as err:
                            str_data = f'参数 {_k} 应该符合 {verify[_k]} 格式'
                    elif verify[_k] == list:
                        # 特殊处理字符串形式 list 类型判断
                        try:
                            checking[_k] = json.loads(_v)
                        except ValueError as err:
                            str_data = f'参数 {_k} 应该符合 {verify[_k]} 格式'
                    else:
                        str_data = f'参数 {_k} 应该是 {verify[_k]} 类型'
                if not null_value and verify[_k] == str and not optional.__contains__(_k):
                    if not _v.strip():
                        str_data = f'参数 {_k} 不能是空字符串'
                if str_data:
                    handle_abnormal(
                        message='请求参数类型校验失败',
                        status=400,
                        other={'prompt': str_data}
                    )
            except KeyError as err:
                if allow_extra:
                    redundant_dict[_k] = _v  # 收集多余的参数
                else:
                    handle_abnormal(
                        message='有多余的请求参数',
                        status=400,
                        other={'prompt': str(err)}
                    )
        if allow_extra:  # 默认不需要收集和返回额外参数
            checking['redundant_dict'] = redundant_dict
        return checking

    def verification_file(self, verify_field: list, verify_suffix: list = []):
        files = self.request.files
        for field in verify_field:
            if not files.get(field, None):
                handle_abnormal(
                    message='表单文件参数不完整',
                    status=400,
                    other={'prompt': verify_field}
                )
        if verify_suffix:
            if len(verify_suffix) != len(verify_field):
                handle_abnormal(
                    message='参数 verify_field 与 verify_suffix 的长度不一致',
                    status=500,
                )
            for i in range(len(verify_suffix)):
                file_name = files[verify_field[i]].filename
                if type(verify_suffix[i]) == list:
                    if file_name.split('.')[-1] not in verify_suffix[i]:
                        handle_abnormal(
                            message='文件字段 {0} 仅支持 {1} 后缀'.format(
                                verify_field[i], verify_suffix[i]),
                            status=400,
                        )
                else:
                    if file_name.split('.')[-1] != verify_suffix[i]:
                        handle_abnormal(
                            message='文件字段 {0} 仅支持 {1} 后缀'.format(
                                verify_field[i], verify_suffix[i]),
                            status=400,
                        )
        return files
